/************************************************************************

 Module:      JTVRemote.cpp

 Project:     JTV

 Description: Aver TVCapture 98 remote control driver. NOT TESTED !
              

 Copyright (c) 1999 by Mirosaw Grybo   <mgrybos@postinfo.com.pl>
 Copyright (c) 2000 by Ivan Kolin  <ivkolin@mail.ru>

 All rights reserved.

 You may not use any piece of the following code for commercial purposes
 without written permission from the author.

 You may freely copy and distribute the source code as long as it contains
 this copyright notice and the disclaimer. You may freely modify it.

 THIS SOFTWARE IS PROVIDED BY THE AUTHOR ``AS IS'' AND ANY EXPRESS OR
 IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
 WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE
 DISCLAIMED.    IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY DIRECT,
 INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES
 (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR
 SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
 HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT,
 STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN
 ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE
 POSSIBILITY OF SUCH DAMAGE.

************************************************************************/
#include <unknwn.h>
#include <stdio.h>

#include "jtvrem.h"
#include "jtvbt8x8.h"
#include "btregs.h"
#include "jtvremsk.h"

// If custom remote have keys listed below, try to avoid modifying
// of key names for JTV compatibility reasons, please!
// It's a good idea to modify only key codes, log-descriptions and
// output file name in order to anyone can keep his customazed
// jtvremote.rm and keys.txt files when driver reselection.
// If possible, try to keep this definition:
//        { 0xCode, "Key_Remote_Pad_Name", "Key_Remote_Alt_Name", },

// Absent remote keys may be remarked or deleted:
//        { 0xffff, "Key_TV", "", },
//        { 0xffff, "Key_FM", "", },   
//        { 0xffff, "Key_Right", "", },
//        { 0xffff, "Key_Left", "", },
//        { 0xffff, "Key_Enter", "", },
//        { 0xffff, "Key_Color", "", },
//        { 0xffff, "Key_MouseMode", "", },
//        { 0xffff, "Key_Cancel", "", },
//        { 0xffff, "Key_AirCableTV", "", },
//        { 0xffff, "Key_Inc", "", },
//        { 0xffff, "Key_AVSource", "", },
//        { 0xffff, "Key_Esc", "", },
      
// BTTool.exe display for "AverMedia TV Capture 98":
// GPIO_Dma_Ctl - 0000c004;
// GPIO_Out_En  - 000078ff;
// GPIO_Data    - remarked column (for pressed -- released
//                keys and closed JTV-window).
// When open JTV-window, GPIO_Data values can change it's
// values in regular way that can be used while driver creation.
// Example: 00xxxxxx - 00xxxxxc, where "x" - random value.
// GPIO_Data    - 00xxxxcc (when JTV-window is open)

KeyDef keys[]=
{ 
// 1-st keys row                            Pressed     Released
        { 0x25bf, "Key_TV/FM", "", },     //0025bfc2 -- 0025bfc2
        { 0x253f, "Key_CD", "", },        //00253fc2 -- 00253fc2
// No key
        { 0x27bf, "Key_Power", "Power", },//0027bfc2 -- ...
// 2-nd keys row
        { 0x1c3f, "Key_1", "1", },        //001c3fc2
        { 0x143f, "Key_2", "2", },        //00143fc2
        { 0x0c3f, "Key_3", "3", },        //000c3fc2
        { 0x043f, "Key_Video", "", },     //00043fc2 
// ...
        { 0x1cbf, "Key_4", "4", },       //001cbfc2
        { 0x14bf, "Key_5", "5", },       //0014bfc2
        { 0x0cbf, "Key_6", "6", },       //000cbfc2
        { 0x04bf, "Key_Audio", "", },    //0004bfc2

        { 0x1d3f, "Key_7", "7", },       //001d3fc2
        { 0x153f, "Key_8", "8", },       //00153fc2
        { 0x0d3f, "Key_9", "9", },       //000d3fc2
        { 0x053f, "Key_FullScreen", "",},//00053fc2

        { 0x1dbf, "Key_0", "0", },        //001dbfc2
        { 0x15bf, "Key_Display", "", },   //0015bfc2
        { 0x0dbf, "Key_Color", "", },     //000dbfc2
        { 0x05bf, "Key_Loop", "", },      //0005bfc2

        { 0x1e3f, "Key_Enter", "Enter", },//001e3fc2
        { 0xd63f, "Key_Left", "Left", },  //00d63fc2---00163fc2
        { 0xce3f, "Key_Right", "Right", },//00ce3fc2---000e3fc2
        { 0x063f, "Key_Preview", "", },   //00063fc2

        { 0x1ebf, "Key_AutoScan", "", },  //001ebfc2
        { 0x16bf, "Key_Freeze", "", },    //0016bfc2
        { 0x0ebf, "Key_Capture", "", },   //000ebfc2
        { 0x06bf, "Key_Mute", "Mute", },  //0006bfc2

        { 0x1f3f, "Key_Red", "", },                //001f3fc2
        { 0xd73f, "Key_VolumeDown", "VolumeDown",},//00d73fc2---00173fc2
        { 0xcf3f, "Key_VolumeUp", "VolumeUp", },   //00cf3fc2---000f3fc2
        { 0x073f, "Key_Yellow", "", },             //00073fc2
 
        { 0x1fbf, "Key_Green", "", },                //001fbfc2
        { 0xd7bf, "Key_ChannelDown", "ChannelDown",},//00d7bfc2---0017bfc2
        { 0xcfbf, "Key_ChannelUp", "ChannelUp",},    //00cfbfc2---000fbfc2
        { 0x07bf, "Key_Cyan", "", },                 //0007bfc2

};

typedef UINT (CALLBACK* LPFNDLLFUNC1)(DWORD,UINT);


DWORD dwParam1;
UINT  uParam2, uReturnVal;


void log(const char* str, ...)
{
  va_list arglist;

  FILE* l=fopen("JTVRemote.log","a+t");
  if (l==NULL) return;
  va_start( arglist, str );
  vfprintf(l, str, arglist );
  va_end( arglist );
  fclose(l);
}

class JTVRemote : public CRemoteDriver
{

public:

        JTVRemote(KeyDef *k, int count) : CRemoteDriver(k, count)
		{
                log("AverC98: Initializing Aver TVCapture98 remote module.        \n");
                log("AverC98: Copyright (c) 2000 by Ivan Kolin, <ivkolin@mail.ru>.\n");
		};


        void PollingThread()
        {
                IBT8x8Device *bt;
                DWORD last_code=0xffffffff;
                DWORD pre_last_code=0xffff;

                if (SUCCEEDED(device->QueryInterface(IID_IBT8x8Device, (void **)&bt)))
                {
                    DWORD value;
                    DWORD key_pause=20; // initial repeat delay

                    bt->ReadBT(BT848_GPIO_DMA_CTL, &value);
                    bt->WriteBT(BT848_GPIO_DMA_CTL, value | 0x0000c004);

                    bt->ReadBT(BT848_GPIO_OUT_EN, &value);
                    bt->WriteBT(BT848_GPIO_OUT_EN, (value | 0x000078ff));

                    bt->ReadBT(BT848_GPIO_DATA, &value);
                    bt->WriteBT(BT848_GPIO_DATA, value | 0x7FC2);


                    while (WaitForSingleObject(hBreakEvent, 0) != WAIT_OBJECT_0)
                    {
                         Sleep(key_pause);

                         DWORD gpio_data=0;
                         DWORD value=0;


                         bt->ReadBT(BT848_GPIO_DATA, &gpio_data);
            		 
                         value = ((gpio_data &0x00ffff00)>>8);
						  						 
                         if ((gpio_data &0x00f00000)>>20 == (0xc|0xd)) // 2-state key is pressed
                         {
                             if (last_code == value)
                             {     // auto repeat for 2-state pressed key
                                   AddStroke(last_code);
                            	   key_pause = 100;
                                   
                             } 
                             else
                             {	   // first run for 2-state pressed key 
                                   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = 1000;
                             }
                             
                         }
                         else if ((gpio_data &0x00f00000)>>20 != (0xc|0xd)) // any other key is pressed
                         {
                             if (last_code == value)
                             {     // auto repeat for 1-state pressed key
                                   AddStroke(last_code);
                            	   key_pause = 100;
                                   
                             } 
                             else
                             {	   // first run for 1-state pressed key 
                                   last_code = value;
                                   AddStroke(last_code);
                                   key_pause = 1000;
                             }
                             	 
                         }
                         else // no one key is pressed
                         {
                             //if ((pre_last_code==0xffff) && (last_code!=0xffffffff))
                             //{
                                last_code=value;
												                              //}
                             // Example_Begin - not used
                             // Released key activation when JTV-window absent
                             // and any key has been pressed before
                             //else if (((gpio_data &0x0000000f)!=0xc) && (last_code!=0xffffffff))
                             //{
                             // 	 AddStroke(value); // Activation for defined RKey_XX
                             //}
                             // Example_End    
                             key_pause = 20;
                         };
				
                    };

                log("AverC98: Break event received. Bye!\n");

                bt->Release();
                };
        };
};

extern "C"
{
        __declspec(dllexport) IJTVRemoteControl *__stdcall GetInterfacePtr()
        {
                return new JTVRemote(keys, sizeof(keys) / sizeof(keys[0]));
        };
};
